#!/bin/sh

# the kernel version and download mirror
kernel_version="2.6.32.40"
kernel_mirror="http://www.us.kernel.org/pub/linux/kernel/v2.6/longterm/v2.6.32"

# the aufs major version
aufs_version=2.1

# fail-safe switch in case someone clicks the script in ROX (real story! not 
# fun at all!!!!) :p
read -p "Press ENTER to begin" dummy

# get the major version (2.6.32 in the case of 2.6.32.40)
kernel_major_version=`echo $kernel_version | cut -f 1-3 -d .`

# get the kernel branch (32 in the case of 2.6.32.40; needed to download Aufs)
kernel_branch=`echo $kernel_major_version | cut -f 3 -d .`

# get the minor version (40 in the case of 2.6.32.40)
case $kernel_version in
	*.*.*.*) kernel_minor_version=`echo $kernel_version | cut -f 4 -d .` ;;
esac

# the package name suffix (-40 in the case of 2.6.32.40); Woof assumes the 
# package version is identical to the kernel version and uses paths that 
# contain the package version, so use $kernel_major version as the version and
# "-$kernel_minor_version" as the suffix
case "$kernel_minor_version" in
	*) package_name_suffix="-$kernel_minor_version";;
esac

# add a special suffix to the package name
package_name_suffix="$package_name_suffix-lts"	
	
# create directories for the results
mkdir -p dist/sources/{patches,vanilla}

# delete the previous log
[ -f build.log ] && rm -f build.log

# download the kernel
if [ ! -f dist/sources/vanilla/linux-$kernel_version.tar.bz2 ]; then
	echo "Downloading the kernel sources"
	wget -P dist/sources/vanilla $kernel_mirror/linux-$kernel_version.tar.bz2 > build.log 2>&1
	if [ $? -ne 0 ]; then
		echo "Error: failed to download the kernel sources."
		exit 1
	fi
fi

# get today's date
today=`date +%d%m%y`

# download Aufs
if [ ! -f dist/sources/vanilla/aufs$aufs_version-$kernel_branch-git$today.tar.bz2 ]; then
	echo "Downloading the Aufs sources"
	git clone http://git.c3sl.ufpr.br/pub/scm/aufs/aufs2-standalone.git aufs$aufs_version-$kernel_branch-git$today >> build.log 2>&1
	if [ $? -ne 0 ]; then
		echo "Error: failed to download the Aufs sources."
		exit 1
	fi
	cd aufs$aufs_version-$kernel_branch-git$today
	git checkout origin/aufs$aufs_version-$kernel_branch >> ../build.log 2>&1
	if [ $? -ne 0 ]; then
		echo "Error: failed to download the Aufs sources."
		exit 1
	fi
	rm -rf .git
	cd ..
	echo "Creating the Aufs sources tarball"
	tar -c aufs$aufs_version-$kernel_branch-git$today | bzip2 -9 > dist/sources/vanilla/aufs$aufs_version-$kernel_branch-git$today.tar.bz2
else
	echo "Extracting the Aufs sources"
	tar xf dist/sources/vanilla/aufs$aufs_version-$kernel_branch-git$today.tar.bz2 >> build.log 2>&1
	if [ $? -ne 0 ]; then
		echo "Error: failed to extract the Aufs sources."
		exit 1
	fi
fi

# patch Aufs
echo "Patching the Aufs sources"
patch -d aufs$aufs_version-$kernel_branch-git$today -p1 < aufs-allow-sfs.patch >> build.log 2>&1
if [ $? -ne 0 ]; then
	echo "Error: failed to patch the Aufs sources."
	exit 1
fi
cp aufs-allow-sfs.patch dist/sources/patches

# extract the kernel
echo "Extracting the kernel sources"
tar xf dist/sources/vanilla/linux-$kernel_version.tar.bz2 >> build.log 2>&1
if [ $? -ne 0 ]; then
	echo "Error: failed to extract the kernel sources."
	exit 1
fi

cd linux-$kernel_version

echo "Adding Aufs to the kernel sources"
for i in kbuild base standalone; do
	patch -p1 < ../aufs$aufs_version-$kernel_branch-git$today/aufs2-$i.patch >> ../build.log 2>&1
	if [ $? -ne 0 ]; then
		echo "Error: failed to add Aufs to the kernel sources."
		exit 1
	fi
done
cp -r ../aufs$aufs_version-$kernel_branch-git$today/{fs,Documentation} .
cp -r ../aufs$aufs_version-$kernel_branch-git$today/include/linux/aufs_type.h include/linux
cat ../aufs$aufs_version-$kernel_branch-git$today/include/linux/Kbuild >> include/linux/Kbuild

echo "Resetting the minor version number"
cp Makefile Makefile-orig
sed -i s/'^EXTRAVERSION = .*$'/'EXTRAVERSION = '/ Makefile
diff -up Makefile-orig Makefile > ../dist/sources/patches/extra-version.patch
rm Makefile-orig

echo "Reducing the number of consoles"
cp kernel/printk.c kernel/printk.c-orig
sed -i s/'#define MAX_CMDLINECONSOLES 8'/'#define MAX_CMDLINECONSOLES 5'/ kernel/printk.c
diff -up kernel/printk.c-orig kernel/printk.c > ../dist/sources/patches/less-consoles.patch

echo "Reducing the verbosity level"
cp -f kernel/printk.c kernel/printk.c-orig
sed -i s/'#define DEFAULT_CONSOLE_LOGLEVEL 7 \/\* anything MORE serious than KERN_DEBUG \*\/'/'#define DEFAULT_CONSOLE_LOGLEVEL 3 \/\* anything MORE serious than KERN_ERR \*\/'/ kernel/printk.c
diff -up kernel/printk.c-orig kernel/printk.c > ../dist/sources/patches/lower-verbosity.patch

for patch in ../patches/*; do
	echo "Applying $patch"
	patch -p1 < $patch >> ../build.log 2>&1
	if [ $? -ne 0 ]; then
		echo "Error: failed to apply $patch on the kernel sources."
		exit 1
	fi
	cp $patch ../dist/sources/patches
done

echo "Cleaning the kernel sources"
make clean
make mrproper
find . -name '*.orig' -delete
find . -name '*.rej' -delete
find . -name '*~' -delete
cp ../DOTconfig .config

mkdir ../dist/packages

echo "Creating the kernel headers package"
make headers_check >> ../build.log 2>&1
make INSTALL_HDR_PATH=kernel_headers-$kernel_major_version-$package_name_suffix headers_install >> ../build.log 2>&1
find kernel_headers-$kernel_major_version-$package_name_suffix/include \( -name .install -o -name ..install.cmd \) -delete
mv kernel_headers-$kernel_major_version-$package_name_suffix ../dist/packages

echo "Compiling the kernel"
make bzImage modules >> ../build.log 2>&1
if [[ ! -f arch/x86/boot/bzImage || ! -f System.map ]]; then
	echo "Error: failed to compile the kernel sources."
	exit 1
fi
cp .config ../dist/sources/DOTconfig-$kernel_version-$today

echo "Creating the kernel package"
make INSTALL_MOD_PATH=linux_kernel-$kernel_major_version-$package_name_suffix modules_install >> ../build.log 2>&1
rm linux_kernel-$kernel_major_version-$package_name_suffix/lib/modules/$kernel_major_version/{build,source}
ln -s $kernel_major_version linux_kernel-$kernel_major_version-$package_name_suffix/lib/modules/$kernel_version
mkdir linux_kernel-$kernel_major_version-$package_name_suffix/boot
cp arch/x86/boot/bzImage linux_kernel-$kernel_major_version-$package_name_suffix/boot/vmlinuz
cp System.map linux_kernel-$kernel_major_version-$package_name_suffix/boot
mv linux_kernel-$kernel_major_version-$package_name_suffix ../dist/packages

echo "Cleaning the kernel sources"
make clean >> ../build.log 2>&1
make prepare >> ../build.log 2>&1

cd ..

echo "Creating a kernel sources SFS"
mkdir -p kernel_sources-$kernel_major_version-$package_name_suffix/usr/src
mv linux-$kernel_version kernel_sources-$kernel_major_version-$package_name_suffix/usr/src/linux
mkdir -p kernel_sources-$kernel_major_version-$package_name_suffix/lib/modules/$kernel_major_version
ln -s /usr/src/linux kernel_sources-$kernel_major_version-$package_name_suffix/lib/modules/$kernel_major_version/build
ln -s /usr/src/linux kernel_sources-$kernel_major_version-$package_name_suffix/lib/modules/$kernel_major_version/source
mksquashfs kernel_sources-$kernel_major_version-$package_name_suffix dist/sources/kernel_sources-$kernel_major_version-$package_name_suffix.sfs

echo "Compressing the log"
bzip2 -9 build.log
cp build.log.bz2 dist/sources

echo "Done!"
aplay /usr/share/sounds/2barks.au
